local Point = sonnet.Point
local List = sonnet.List
local Tween = sonnet.Tween
local Clock = sonnet.Clock
Tower = sonnet.middleclass.class('Tower')

function Tower:initialize(game, loc)
    self.game = game
    self.map = game.map
    self.loc = loc

    self.angle = math.random(360) / 180 * math.pi
    self.shot_clock = Clock(2, self.shoot, self)
    self.target = false

    self.range = 96
    self.damage = 20
    self.level = 1
end

function Tower:can_upgrade()
    return self.level < 5
end

function Tower:upgrade_cost()
    return self.level * 10
end

function Tower:upgrade(type)
    self.level = self.level + 1
    if type == 'range' then
        self.range = self.range + 24
    elseif type == 'damage' then
        self.damage = self.damage + 20
    elseif type == 'fire' then
        self.shot_clock.delay = self.shot_clock.delay - 0.25
    end
end

function Tower:caption()
    return string.format("Tower level %d / 5\nRange %d / Dmg %d / Spd %s",
                         self.level, self.range / 24, self.damage, self.shot_clock.delay)
end

function Tower:update(dt)
    if self.target and not self:valid_target() then
        self.target = nil
    end

    if not self.target then
        self:choose_target()
    end

    if self.target then
        local center = self.loc * 24 + Point(12, 12)
        self.angle = math.atan2(self.target.loc.y-center.y,
                                self.target.loc.x-center.x)
    end
end

-- ## valid target
--
-- Returns true if the current target exists, is
-- alive, and is in range.

function Tower:valid_target()
    if not self.target then return false end
    if not self.target:is_alive() then return false end
    if self.target.health <= 0 then return false end
    local center = self.loc*24 + Point(12, 12)
    if not center:dist(self.target.loc, self.range) then
        return false end
    return true
end

-- ## choose target
--
-- Set target to the closest Bug in range

function Tower:choose_target()
    local closest, closest_dist = nil
    local center = self.loc * 24 + Point(12, 12)
    for _, bug in self.game.bugs:each() do
        local dist = center:dist(bug.loc)
        if (not closest and dist <= self.range or
            closest and dist < closest_dist) and bug.health > 0 then
            closest = bug
            closest_dist = dist
        end
    end

    if closest then self.target = closest end
end

function Tower:draw()
    local g = love.graphics
    g.push()
    g.translate(self.loc.x*24, self.loc.y*24)

    g.setColor(140, 140, 160, 255)
    g.rectangle('fill', 0, 0, 24, 24)

    g.setLineWidth(2)
    g.setColor(255, 255, 255, 255)
    g.rectangle('line', 0, 0, 24, 24)

    g.line(12, 12,
           12+math.cos(self.angle)*12,
           12+math.sin(self.angle)*12)

    g.pop()
end

function Tower:shoot()
    if not self.target then return end

    local barrel = Point(math.cos(self.angle),
                         math.sin(self.angle))*10

    local x, y = (self.loc*24+Point(12, 12)+barrel)()

    local b = sonnet.effects.Bullet(Point(x,y),self.target.loc,250)
    sonnet.effects.Spray(x, y, self.angle)

    b:promise():add(function()
                        if self.target then
                            self.target.health =
                                self.target.health - self.damage
                        end
                    end)
end